# Basic constructs:
from ProductRegisters.FeedbackRegister import FeedbackRegister
from ProductRegisters.FeedbackFunctions import *

# Boolean logic and chaining templates
from ProductRegisters.BooleanLogic import *
from ProductRegisters.BooleanLogic.ChainingGeneration.Templates import *
import ProductRegisters.BooleanLogic.ChainingGeneration.TemplateBuilding

# Berlekamp-Massey and variants
from ProductRegisters.Tools.RegisterSynthesis.lfsrSynthesis import *
from ProductRegisters.Tools.RegisterSynthesis.fcsrSynthesis import *
from ProductRegisters.Tools.RegisterSynthesis.nlfsrSynthesis import *

# Tools and other extraneous files
import ProductRegisters.Tools.ResolventSolving as ResolventSolving
from ProductRegisters.Tools.RootCounting.MonomialProfile import *

# Cryptanalysis:
from ProductRegisters.Cryptanalysis.utility import *

# MPR descriptions
# The first polynomial is the primitive feedback polynomial, which will not be changed
# The second polynomial is the update polynomial, which you will be changing

# Different update polynomials to test: x, x^2, x^3, x^4, x^5, x^15

M127 = MPR(127, poly("1 + x^13 + x^45 + x^54 + x^127"), poly("x")) # You will be changing this update polynomial only

M19 = MPR(19, poly("1 + x + x^2 + x^3 + x^4 + x^5 + x^19"), poly("x"))

M17 = MPR(17, poly("1 + x + x^2 + x^3 + x^17"), poly("x"))

M5 = MPR(5, poly("1 + x^2 + x^3 + x^4 + x^5"), poly("x"))

M2 = MPR(2, poly("1 + x + x^2"), poly("x"))

# CMPR: 170 bits = 127 + 19 + 17 + 5 + 2
C170 = CMPR([M127, M19, M17, M5, M2])
C170.generateChaining(template=old_ANF_template(max_and=4, max_xor=4)) # Generate nonlinear boolean logic for the feedback register

# VHDL Generation
C170.write_VHDL("C170.vhd")

print("Done.")